@@ -15,6 +15,11 @@
 # specific language governing permissions and limitations
 # under the License.
 
+import datetime
+import os
+import re
+import sys
+
 # Configuration file for the Sphinx documentation builder.
 #
 # This file only contains a selection of the most common options. For a full
@@ -26,17 +31,24 @@
 # If extensions (or modules to document with autodoc) are in another directory,
 # add these directories to sys.path here. If the directory is relative to the
 # documentation root, use os.path.abspath to make it absolute, like shown here.
-#
-import os
-import sys
-import datetime
-sys.path.insert(0, os.path.abspath('..'))
+sys.path.insert(0, os.path.abspath(".."))
 
 # -- Project information -----------------------------------------------------
 
-project = 'nanoarrow'
-copyright = f'2016-{datetime.datetime.now().year} Apache Software Foundation'
-author = 'Apache Software Foundation'
+project = "nanoarrow"
+copyright = f"2016-{datetime.datetime.now().year} Apache Software Foundation"
+author = "Apache Software Foundation"
+
+
+def get_version():
+    cmakelists = os.path.join("..", "..", "CMakeLists.txt")
+    with open(cmakelists, "r") as f:
+        for line in f:
+            if "NANOARROW_VERSION" in line:
+                if "SNAPSHOT" in line:
+                    return "dev"
+                else:
+                    return re.findall(r"[0-9]+\.[0-9]+\.[0-9]+", line)[0]
 
 
 # -- General configuration ---------------------------------------------------
@@ -45,24 +57,25 @@ author = 'Apache Software Foundation'
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
 # ones.
 extensions = [
-    'breathe',
-    'sphinx.ext.autodoc',
+    "breathe",
+    "sphinx.ext.autodoc",
 ]
 
 # Breathe configuration
 breathe_projects = {
     "nanoarrow_c": "../../src/apidoc/xml",
-    "nanoarrow_ipc": "../../extensions/nanoarrow_ipc/src/apidoc/xml"
+    "nanoarrow_ipc": "../../extensions/nanoarrow_ipc/src/apidoc/xml",
+    "nanoarrow_device": "../../extensions/nanoarrow_device/src/apidoc/xml",
 }
 breathe_default_project = "nanoarrow_c"
 
 # Add any paths that contain templates here, relative to this directory.
-templates_path = ['_templates']
+templates_path = ["_templates"]
 
 # List of patterns, relative to source directory, that match files and
 # directories to ignore when looking for source files.
 # This pattern also affects html_static_path and html_extra_path.
-exclude_patterns = ['README_generated.rst']
+exclude_patterns = ["README_generated.rst"]
 
 
 # -- Options for HTML output -------------------------------------------------
@@ -70,14 +83,17 @@ exclude_patterns = ['README_generated.rst']
 # The theme to use for HTML and HTML Help pages.  See the documentation for
 # a list of builtin themes.
 #
-html_theme = 'pydata_sphinx_theme'
+html_theme = "pydata_sphinx_theme"
 
 html_theme_options = {
     "show_toc_level": 2,
     "use_edit_page_button": True,
-    "external_links": [
-      {"name": "R Package", "url": "r/index.html"},
-  ],
+    "external_links": [],
+    "switcher": {
+        "json_url": "https://arrow.apache.org/nanoarrow/switcher.json",
+        "version_match": get_version(),
+    },
+    "navbar_start": ["navbar-logo", "version-switcher"],
 }
 
 html_context = {
@@ -87,9 +103,7 @@ html_context = {
     "doc_path": "docs/source",
 }
 
-html_sidebars = {
-    "**": ["search-field", "sidebar-nav-bs"]
-}
+html_sidebars = {"**": ["search-field", "sidebar-nav-bs"]}
 
 
 # Add any paths that contain custom static files (such as style sheets) here,
