@@ -271,6 +271,104 @@ TEST(BitmapTest, BitmapTestElement) {
   EXPECT_EQ(ArrowBitGet(bitmap, 16 + 7), 0);
 }
 
+template <int offset, int length>
+void TestArrowBitmapUnpackUnsafe(const uint8_t* bitmap, std::vector<int8_t> expected) {
+  int8_t out[length];
+  int32_t out32[length];
+  memset(out, 0, sizeof(out));
+  memset(out32, 0, sizeof(out32));
+
+  ASSERT_EQ(length, expected.size());
+
+  ArrowBitsUnpackInt8(bitmap, offset, length, out);
+  for (int i = 0; i < length; i++) {
+    EXPECT_EQ(out[i], expected[i]);
+  }
+
+  ArrowBitsUnpackInt32(bitmap, offset, length, out32);
+  for (int i = 0; i < length; i++) {
+    EXPECT_EQ(out32[i], expected[i]);
+  }
+}
+
+TEST(BitmapTest, BitmapTestBitmapUnpack) {
+  uint8_t bitmap[3];
+  int8_t result[sizeof(bitmap) * 8];
+  int32_t result32[sizeof(result)];
+  int64_t n_values = sizeof(result);
+
+  // Basic test of a validity buffer that is all true
+  memset(bitmap, 0xff, sizeof(bitmap));
+  memset(result, 0, sizeof(result));
+  memset(result32, 0, sizeof(result32));
+
+  ArrowBitsUnpackInt8(bitmap, 0, sizeof(result), result);
+  for (int i = 0; i < n_values; i++) {
+    EXPECT_EQ(result[i], 1);
+  }
+
+  ArrowBitsUnpackInt32(bitmap, 0, sizeof(result), result32);
+  for (int i = 0; i < n_values; i++) {
+    EXPECT_EQ(result32[i], 1);
+  }
+
+  // Ensure that the first byte/middle byte/last byte logic is correct
+  // Note that TestArrowBitmapUnpack tests both the int8 and int32 version
+  bitmap[0] = 0x93;  // 10010011
+  bitmap[1] = 0x55;  // 01010101
+  bitmap[2] = 0xaa;  // 10101010
+
+  // offset 0, length boundary, one byte
+  TestArrowBitmapUnpackUnsafe<0, 8>(bitmap, {1, 1, 0, 0, 1, 0, 0, 1});
+
+  // offset 0, length boundary, different bytes
+  TestArrowBitmapUnpackUnsafe<0, 16>(bitmap,
+                                     {1, 1, 0, 0, 1, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0});
+
+  // offset 0, length non-boundary, one byte
+  TestArrowBitmapUnpackUnsafe<0, 5>(bitmap, {1, 1, 0, 0, 1});
+
+  // offset boundary, length boundary, one byte
+  TestArrowBitmapUnpackUnsafe<8, 8>(bitmap, {1, 0, 1, 0, 1, 0, 1, 0});
+
+  // offset boundary, length boundary, different bytes
+  TestArrowBitmapUnpackUnsafe<8, 16>(bitmap,
+                                     {1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1});
+
+  // offset boundary, length non-boundary, one byte
+  TestArrowBitmapUnpackUnsafe<8, 5>(bitmap, {1, 0, 1, 0, 1});
+
+  // offset boundary, length non-boundary, different bytes
+  TestArrowBitmapUnpackUnsafe<8, 13>(bitmap, {1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0});
+
+  // offset non-boundary, length boundary, one byte
+  TestArrowBitmapUnpackUnsafe<3, 5>(bitmap, {0, 1, 0, 0, 1});
+
+  // offset non-boundary, length boundary, different bytes
+  TestArrowBitmapUnpackUnsafe<3, 13>(bitmap, {0, 1, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0});
+
+  // offset non-boundary, length non-boundary, one byte
+  TestArrowBitmapUnpackUnsafe<3, 3>(bitmap, {0, 1, 0});
+
+  // offset non-boundary, length non-boundary, different bytes
+  TestArrowBitmapUnpackUnsafe<3, 11>(bitmap, {0, 1, 0, 0, 1, 1, 0, 1, 0, 1, 0});
+
+  // offset non-boundary non-first byte, length boundary, one byte
+  TestArrowBitmapUnpackUnsafe<11, 5>(bitmap, {0, 1, 0, 1, 0});
+
+  // offset non-boundary non-first byte, length boundary, different bytes
+  TestArrowBitmapUnpackUnsafe<11, 13>(bitmap, {0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1});
+
+  // offset non-boundary non-first byte, length non-boundary, one byte
+  TestArrowBitmapUnpackUnsafe<11, 3>(bitmap, {0, 1, 0});
+
+  // offset non-boundary non-first byte, length non-boundary, different bytes
+  TestArrowBitmapUnpackUnsafe<11, 11>(bitmap, {0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1});
+
+  // non-boundary, three byte span
+  TestArrowBitmapUnpackUnsafe<7, 11>(bitmap, {1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1});
+}
+
 TEST(BitmapTest, BitmapTestSetTo) {
   uint8_t bitmap[10];
 
